#include <windows.h>
#include <stdio.h>
// For _getch()
#include <conio.h>

// Global variable
HANDLE hStdout; 

int ScrollByAbsoluteCoord(int iRows)
{
    CONSOLE_SCREEN_BUFFER_INFO csbiInfo; 
    SMALL_RECT srctWindow; 
 
    // Get the current screen buffer size and window position. 
    if (!GetConsoleScreenBufferInfo(hStdout, &csbiInfo)) 
    {
        wprintf(L"GetConsoleScreenBufferInfo() failed, error %d\n", GetLastError()); 
        return 1;
    }
 
    // Set srctWindow to the current window size and location. 
    srctWindow = csbiInfo.srWindow; 
 
    // Check whether the window is too close to the screen buffer top 
    if (srctWindow.Top >= iRows) 
    { 
        srctWindow.Top -= iRows;     // move top up
        srctWindow.Bottom -= iRows;  // move bottom up

        if (!SetConsoleWindowInfo( 
                   hStdout,          // screen buffer handle 
                   TRUE,             // absolute coordinates 
                   &srctWindow))     // specifies new location 
        {
            wprintf(L"SetConsoleWindowInfo() failed, error %d\n", GetLastError()); 
            return 1;
        }
        return iRows;
    }
    else
    {
        wprintf(L"\nCannot scroll; the window is too close to the top.\n");
        return 1;
    }
}

int ScrollByRelativeCoord(int iRows)
{
    CONSOLE_SCREEN_BUFFER_INFO csbiInfo; 
    SMALL_RECT srctWindow; 

    // Get the current screen buffer window position. 
    if (!GetConsoleScreenBufferInfo(hStdout, &csbiInfo)) 
    {
        wprintf(L"GetConsoleScreenBufferInfo(), error %d\n", GetLastError()); 
        return 1;
    }
 
    // Check whether the window is too close to the screen buffer top 
    if (csbiInfo.srWindow.Top >= iRows)
    {
        srctWindow.Top = -iRows;     // move top up
        srctWindow.Bottom = -iRows;  // move bottom up 
        srctWindow.Left = 0;         // no change 
        srctWindow.Right = 0;        // no change 
 
        if (!SetConsoleWindowInfo( 
                   hStdout,          // screen buffer handle 
                   FALSE,            // relative coordinates
                   &srctWindow))     // specifies new location 
        {
            wprintf(L"SetConsoleWindowInfo(), error %d\n", GetLastError()); 
            return 0;
        }
        return iRows;
    }
    else
    {
        wprintf(L"\nCannot scroll; the window is too close to the top.\n");
        return 0;
    }
}

int wmain(int argc, WCHAR **argv)
{
    int i;

    wprintf(L"\nPrinting sixty lines, then scrolling up five lines.\n");
    wprintf(L"Press any key to scroll up ten lines...\n");
    wprintf(L"then press another key to stop the demo.\n");

    for(i=0; i<=60; i++)
        wprintf(L"%d\n", i);

    hStdout = GetStdHandle(STD_OUTPUT_HANDLE); 

    if(ScrollByAbsoluteCoord(5))
	{
		wprintf(L" Scrolling up by 5...\n");
		_getwch();
	}
    else
		return 1;

    if(ScrollByRelativeCoord(10))
	{
		wprintf(L" Scrolling up by 10...\n");
		_getwch();
	}
    else
		return 1;

	return 0;
}
